# -*- coding: utf-8 -*-

# Copyright (C) 2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
# This file is part of minicom-webadmin
# minicom-webadmin is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.contrib.auth.decorators import login_required

from webadmin.system.models import Info
from webadmin.firewall.models import Firewall
from webadmin.firewall.forms import FormFirewall, FormLog
from webadmin.tools import System, Task

def update_context():
    """
    Update the context dictonary with
    commom informations to be rendered in the HTML template
    """
    context = {'modulo': 'Bloqueios',
               'extra': 'Sites',
               }
    try:
        blocked_urls = Firewall.objects.all()
        context.update({'urls': blocked_urls})
    except Firewall.DoesNotExist:
        pass

    return context

@login_required
def block_sites(request):
    """
    Block a url with a given keyword.
    """
    registered = Info.objects.all()
    context = {'icon': 'web-security.png'}
    if request.method == 'POST':
        form = FormFirewall(request.POST, request.FILES)
        context.update(update_context())
        if form.is_valid():
            rules = form.save(commit=False)
            url = form.cleaned_data['url']

            task = Task()
            status = task.blockUrl(url, True)
            if status == 'Permission denied' or status == 0:
                if status == 0:
                    errmsg = u'Arquivo de regras do squid inexistente!'
                else:
                    errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg})
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
            else:
                rules.save()
                mensagem = u'Regra %s adicionada a lista de bloqueios.' % url
                context.update({'msg': mensagem})
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
        return render_to_response('system/security/firewall.html', context,
                                  context_instance=RequestContext(request))
    else:
        form = FormFirewall()
        context = update_context()
        context.update({'modulo': 'Bloqueios',
                        'extra': 'Sites',
                        'icon': 'web-security.png',
                        'form': form,
                        })
        if not registered:
            return render_to_response('system/warning.html', context,
                                      context_instance=RequestContext(request))
        else:
            return render_to_response('system/security/firewall.html', context,
                                      context_instance=RequestContext(request))

@login_required
def alter_site(request, id_rule):
    """
    Modify an existent keyword.
    """
    context = {'icon': 'web-security.png'}
    site = get_object_or_404(Firewall, pk=id_rule)
    form = FormFirewall({'url': site})

    if request.method == 'POST':
        form = FormFirewall(request.POST, instance=site)
        if form.is_valid():
            site = form.save(commit=False)
            new_site_name = form.cleaned_data['url']
            site.url = new_site_name
            task = Task()
            ret = task.blockUrl(site.url, True)
            if ret != 1:
                errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
                context.update({'errmsg': errmsg})
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
            else:
                form.save()
                message = u'Regra %s alterada com sucesso.' % new_site_name
                context.update({'msg': message})
                context.update(update_context())
                return render_to_response('system/security/firewall.html', context,
                                          context_instance=RequestContext(request))
    else:
        form = FormFirewall(instance=site)
        context.update({'form': form})

    context.update(update_context())
    return render_to_response('system/security/firewall.html', context,
                              context_instance=RequestContext(request))

@login_required
def remove_site(request, id_rule):
    """
    Remove a keyword from the denied list of Squid.
    """
    context = {'icon': 'web-security.png'}
    context.update(update_context())
    try:
        site = Firewall.objects.get(id=id_rule)
        task = Task()
        ret = task.blockUrl(site.url, False)
        if ret != 1:
            errmsg = u'Sem permissão de escrita no arquivo de tarefas!'
            context.update({'errmsg': errmsg})
            return render_to_response('system/security/firewall.html', context,
                                      context_instance=RequestContext(request))
        else:
            site.delete()
            message = u'Página %s removida da lista de bloqueios.' % site.url
            context.update({'msg': message})
            context.update(update_context())
            return redirect('/system/security/')
            #return render_to_response('system/security/firewall.html', context,
            #                          context_instance=RequestContext(request))
    except Firewall.DoesNotExist:
        pass
    
    return render_to_response('system/security/firewall.html', {},
                              context_instance=RequestContext(request))

@login_required
def log(request):
    """
    Reads the log with only attempts denied by Squid.    
    """

    context = {'modulo': 'Consulta log',
               'extra': 'Squid',
               'icon': 'filefind.png',
               }

    logfile = '/var/log/squid3/access.log'
    form = FormLog()

    def getLog(num_entries):
        """
        Read the log and filter the number of entries
        """

        task = Task()
        log = task.readLog(num_entries, logfile)
        # log is a tuple
        # log[0] = status
        # log[1] = logfile lines or error message, if status is 1 or 0 respectively.
        if log[0]:
            context.update({'form': form,
                            'lines': log[1]})
        else:
            errmsg = u'Sem permissão de leitura no arquivo %s' % logfile
            context.update({'errmsg': errmsg})
        return context

    context = getLog(50)

    if request.method == 'POST':
        form = FormLog(request.POST)
        if form.is_valid():
            num_entries = form.cleaned_data['num_entries']
            if num_entries:
                context.update(getLog(num_entries))
            else:
                context.update(getLog(50))
            return render_to_response('system/security/log.html', context,
                                      context_instance=RequestContext(request))
    else:
        form = FormLog()
        return render_to_response('system/security/log.html', context,
                                  context_instance=RequestContext(request))
